@@ -1,7 +1,7 @@
 # This script generates the PyQt configuration and generates the Makefiles.
 #
 # Copyright (c) 2007
-# 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
+# 	Phil Thompson <phil@river-bank.demon.co.uk>
 # 
 # This file is part of PyQt.
 # 
@@ -31,10 +31,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040300
-pyqt_version_str = "4.3"
+pyqt_version = 0x040303
+pyqt_version_str = "4.3.3"
 
-sip_min_version = 0x040700
+sip_min_version = 0x040702
 
 qt_version = 0
 qt_edition = ""
@@ -46,7 +46,6 @@ qt_datadir = None
 qt_pluginsdir = None
 qt_xfeatures = None
 qt_shared = ""
-qt_lib = None
 qt_framework = 0
 
 qt_sip_flags = []
@@ -117,12 +116,12 @@ def create_optparser():
 
     # Note: we don't use %default to be compatible with Python 2.3.
     p.add_option("-k", "--static", action="store_true", default=False,
-            help="build modules as static libraries")
+            dest="static", help="build modules as static libraries")
     p.add_option("-r", "--trace", action="store_true", default=False,
             dest="tracing", help="build modules with tracing enabled")
     p.add_option("-u", "--debug", action="store_true", default=False,
             help="build modules with debugging symbols")
-    p.add_option("-w", "--verbose", action="count", default=0,
+    p.add_option("-w", "--verbose", action="count", default=0, dest="verbose",
             help="verbose output during configuration")
 
     p.add_option("-c", "--concatenate", action="store_true", default=False,
@@ -136,6 +135,8 @@ def create_optparser():
             "all the Qt libraries")
 
     # These are internal options used to build the mega Windows GPL package.
+    p.add_option("--mwg-odbc", action="store_true", default=False,
+            dest="mwg_odbc", help=optparse.SUPPRESS_HELP)
     p.add_option("--mwg-openssl", action="callback", default=None,
             dest="mwg_ssl_dir", metavar="DIR", callback=store_abspath_dir,
             type="string", help=optparse.SUPPRESS_HELP)
@@ -148,6 +149,8 @@ def create_optparser():
 
     # Configuration.
     g = optparse.OptionGroup(p, title="Configuration")
+    g.add_option("--confirm-license", action="store_true", default=False,
+            dest="license_confirmed", help="confirm acceptance of the license")
     g.add_option("-e", "--enable", action="append", default=[],
             metavar="MODULE", dest="enabled", help="enable checks for the "
             "specified MODULE [default: checks for all modules will be "
@@ -158,7 +161,8 @@ def create_optparser():
 
     if sys.platform != "win32":
         g.add_option("-q", "--qmake", action="callback", metavar="FILE",
-                default=qmake, callback=store_abspath_file, type="string",
+                default=qmake, dest="qmake", callback=store_abspath_file,
+                type="string",
                 help="the pathname of qmake [default: %s]" % (qmake or "none"))
 
     g.add_option("-s", "--dbus", action="callback", metavar="DIR",
@@ -245,10 +249,23 @@ class ConfigurePyQt4:
             0x040200: "Qt_4_1_3",
             0x040202: "Qt_4_2_0",
             0x040300: "Qt_4_2_2",
-            0x050000: "Qt_4_3_0"
+            0x040303: "Qt_4_3_0",
+            0x050000: "Qt_4_3_3"
         }
 
     def check_modules(self):
+        if opts.mwg_odbc:
+            sql_libs = ["odbc32"]
+        else:
+            sql_libs = None
+
+        if opts.mwg_ssl_dir:
+            ass_lib_dirs = [os.path.join(opts.mwg_ssl_dir, "lib")]
+            ass_libs = ["ssleay32", "libeay32"]
+        else:
+            ass_lib_dirs = None
+            ass_libs = None
+
         # Note that the order in which we check is important for the
         # consolidated module - a module's dependencies must be checked first.
         pyqt_modules.append("QtCore")
@@ -257,19 +274,14 @@ class ConfigurePyQt4:
         check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
         check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
         check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
-        check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()")
+        check_module("QtSql", "qsqldatabase.h", "new QSqlDatabase()",
+                extra_libs=sql_libs)
         check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
         check_module("QtTest", "QtTest", "QTest::qSleep(0)")
         check_module("QtXml", "qdom.h", "new QDomDocument()")
-
-        if opts.mwg_ssl_dir:
-            check_module("QtAssistant", "qassistantclient.h",
-                    "new QAssistantClient(\"foo\")",
-                    extra_lib_dirs=[os.path.join(opts.mwg_ssl_dir, "lib")],
-                    extra_libs=["ssleay32", "libeay32"])
-        else:
-            check_module("QtAssistant", "qassistantclient.h",
-                    "new QAssistantClient(\"foo\")")
+        check_module("QtAssistant", "qassistantclient.h",
+                "new QAssistantClient(\"foo\")", extra_lib_dirs=ass_lib_dirs,
+                extra_libs=ass_libs)
 
         if not qt_shared:
             sipconfig.inform("QtDesigner module disabled with static Qt libraries.")
@@ -459,6 +471,9 @@ class ConfigurePyQt4:
 
             f.close()
 
+            if opts.mwg_odbc:
+                cons_xtra_libs.append("odbc32")
+
             if opts.mwg_ssl_dir:
                 cons_xtra_libdirs.append(os.path.join(opts.mwg_ssl_dir, "lib"))
                 cons_xtra_libs.extend(["ssleay32", "libeay32"])
@@ -788,7 +803,7 @@ def inform_user():
 
     sipconfig.inform("SIP %s is being used." % sipcfg.sip_version_str)
     sipconfig.inform("The Qt header files are in %s." % qt_incdir)
-    sipconfig.inform("The %s %s library is in %s." % (lib_type, qt_lib, qt_libdir))
+    sipconfig.inform("The %s Qt libraries are in %s." % (lib_type, qt_libdir))
     sipconfig.inform("The Qt binaries are in %s." % qt_bindir)
     sipconfig.inform("The Qt mkspecs directory is in %s." % qt_datadir)
     sipconfig.inform("These PyQt modules will be built: %s." % string.join(pyqt_modules))
@@ -1299,40 +1314,36 @@ def check_license():
     sipconfig.inform("This is the %s version of PyQt %s (licensed under the %s) for Python %s on %s." % (ltype, pyqt_version_str, lname, string.split(sys.version)[0], sys.platform))
 
     # Common checks.
-    try:
-        qted = qt_edition
-    except AttributeError:
-        qted = None
+    if qt_edition and ltype != "internal":
+        if (qt_edition == "free" and ltype != "GPL") or (qt_edition != "free" and ltype == "GPL"):
+            sipconfig.error("This version of PyQt and the %s edition of Qt have incompatible licenses." % qt_edition)
 
-    if qted and ltype != "internal":
-        if (qted == "free" and ltype != "GPL") or (qted != "free" and ltype == "GPL"):
-            sipconfig.error("This version of PyQt and the %s edition of Qt have incompatible licenses." % qted)
+    # Confirm the license if not already done.
+    if not opts.license_confirmed:
+        print
+        print "Type 'L' to view the license."
+        print "Type 'yes' to accept the terms of the license."
+        print "Type 'no' to decline the terms of the license."
+        print
 
-    # Confirm the license.
-    print
-    print "Type 'L' to view the license."
-    print "Type 'yes' to accept the terms of the license."
-    print "Type 'no' to decline the terms of the license."
-    print
+        while 1:
+            try:
+                resp = raw_input("Do you accept the terms of the license? ")
+            except KeyboardInterrupt:
+                raise SystemExit
+            except:
+                resp = ""
 
-    while 1:
-        try:
-            resp = raw_input("Do you accept the terms of the license? ")
-        except KeyboardInterrupt:
-            raise SystemExit
-        except:
-            resp = ""
+            resp = string.lower(string.strip(resp))
 
-        resp = string.lower(string.strip(resp))
+            if resp == "yes":
+                break
 
-        if resp == "yes":
-            break
+            if resp == "no":
+                sys.exit(0)
 
-        if resp == "no":
-            sys.exit(0)
-
-        if resp == "l":
-            os.system("more LICENSE")
+            if resp == "l":
+                os.system("more LICENSE")
 
     # If there should be a license file then check it is where it should be.
     if lfile:
@@ -1380,62 +1391,16 @@ def get_build_macros(overrides):
     return sipconfig.parse_build_macros(fname, names, overrides, properties)
 
 
-def qt_core_header(incdir, fname):
-    """Find a QtCore header file.
-
-    incdir is the directory expected to contain the file.
-    fname is the base name of the file.
-
-    Returns the full path of the file or None if it wasn't found.
-    """
-    pn = os.path.join(incdir, fname)
-
-    try:
-        sbuf = os.stat(pn)
-    except OSError:
-        sbuf = None
-
-    if sbuf:
-        # In some Qt4 installations the public header files just have a single
-        # line that #includes the real file in the src directory.  Rather than
-        # parse #include directives we just look at the size of the file and
-        # assume it contains a #include if it seems too small to contain
-        # anything useful.
-        if sbuf.st_size < 100:
-            sn = os.path.join(qt_dir, "src", "corelib", "global", fname)
-
-            if os.access(sn, os.F_OK):
-                pn = sn
-    else:
-        pn = None
-
-    return pn
-
-
-def qt_inc_dir(qtmod):
-    """Get the directory containing the header files for a particular Qt
-    module.
-
-    qtmod is the name of the Qt module.
-
-    Returns the directory name.
-    """
-    # QLibraryInfo doesn't report the correct directories when built as a
-    # Darwin framework.
-    if qt_framework:
-        incdir = os.path.join(qt_libdir, qtmod + ".framework", "Headers")
-    else:
-        incdir = os.path.join(qt_incdir, qtmod)
-
-    return incdir
-
-
 def check_qt_installation(macros):
     """Check the Qt installation and get the version number and edition and
     return the configuration instance.
 
     macros is the dictionary of build macros.
     """
+    # Check the Qt version number.
+    if qt_version < 0x040100:
+        sipconfig.error("PyQt v4 requires Qt v4.1.0 or later.")
+
     # Work out how Qt was built on MacOS.
     if sys.platform == "darwin":
         if os.access(os.path.join(qt_libdir, "QtCore.framework"), os.F_OK):
@@ -1445,64 +1410,6 @@ def check_qt_installation(macros):
     # Get the Makefile generator.
     generator = macros["MAKEFILE_GENERATOR"]
 
-    # Check the Qt header files have been installed.
-    incdir = qt_inc_dir("QtCore")
-
-    qglobal = qt_core_header(incdir, "qglobal.h")
-
-    if not qglobal:
-        sipconfig.error("qglobal.h could not be found in %s." % incdir)
-
-    # Get the Qt version number.
-    global qt_version
-
-    qt_version, ignore = sipconfig.read_version(qglobal, "Qt", "QT_VERSION")
-
-    if qt_version < 0x040100:
-        sipconfig.error("PyQt v4 requires Qt v4.1.0 or later.")
-
-    # Try and work out which edition it is.
-    global qt_edition
-
-    # See if it is the open source edition.
-    qconfig = qt_core_header(incdir, "qconfig.h")
-
-    if not qconfig:
-        sipconfig.error("qconfig.h could not be found in %s." % incdir)
-
-    f = open(qconfig)
-    l = f.readline()
-
-    while l:
-        wl = string.split(l)
-
-        if len(wl) == 3 and wl[0] == "#define":
-            name_val = wl[1:]
-        elif len(wl) == 4 and wl[0] == "#" and wl[1] == "define":
-            name_val = wl[2:]
-        else:
-            name_val = None
-
-        if name_val and name_val[0] == "QT_EDITION" and name_val[1] == "QT_EDITION_OPENSOURCE":
-            qt_edition = "free"
-            break
-
-        l = f.readline()
-
-    f.close()
-
-    if not qt_edition:
-        # Work out which commercial edition it is.
-        if not os.access(qt_inc_dir("QtGui"), os.F_OK):
-            qt_edition = "Console"
-        elif not os.access(qt_inc_dir("QtXml"), os.F_OK):
-            qt_edition = "Desktop Light"
-        else:
-            qt_edition = "Desktop"
-
-    # Determine the Qt library to link against.
-    resolve_qt4_library(generator)
-
     # We haven't yet factored out sipconfig's knowledge of how to build Qt
     # binaries and it is expecting to find these in the configuration when it
     # generates the Makefiles.
@@ -1517,49 +1424,6 @@ def check_qt_installation(macros):
     return ConfigurePyQt4(generator)
 
 
-def resolve_qt4_library(generator):
-    """Check that the Qt v4 library can be found.
-
-    generator is the name of the Makefile generator.
-    """
-    global qt_lib
-
-    qt_lib = "QtCore"
-
-    if opts.debug and sys.platform != "win32":
-        # Qt v4.2 and later for X11 platforms doesn't have separate debug
-        # libraries.
-        if qt_version < 0x040200 or sys.platform == "darwin":
-            qt_lib = qt_lib + "_debug"
-
-    if not is_qt_library(generator, qt_lib):
-        sipconfig.error("The %s Qt library could not be found in %s." % (qt_lib, qt_libdir))
-
-
-def is_qt_library(generator, lib):
-    """See if a particular Qt library is installed.
-
-    generator is the name of the Makefile generator.
-    lib is the name of the library.
-    """
-    if generator in ("MSVC", "MSVC.NET", "BMAKE"):
-        lpatts = [lib + "[0-9]*.lib", lib + ".lib"]
-    elif generator == "MINGW":
-        lpatts = ["lib" + lib + "[0-9]*.a"]
-    elif sys.platform == "darwin" and qt_framework:
-        lpatts = [lib + ".framework"]
-    else:
-        lpatts = ["lib" + lib + ".*"]
-
-    for lpatt in lpatts:
-        lmatch = glob.glob(os.path.join(qt_libdir, lpatt))
-
-        if lmatch:
-            return lmatch
-
-    return []
-
-
 def fix_qmake_args(args=""):
     """Make any platform specific adjustments to the arguments passed to qmake.
     """
@@ -1638,6 +1502,9 @@ int main(int, char **)
     out << QLibraryInfo::location(QLibraryInfo::DataPath) << '\\n';
     out << QLibraryInfo::location(QLibraryInfo::PluginsPath) << '\\n';
 
+    out << QT_VERSION << '\\n';
+    out << QT_EDITION << '\\n';
+
 #if defined(QT_SHARED) || defined(QT_DLL)
     out << "shared\\n";
 #else
@@ -1700,7 +1567,7 @@ int main(int, char **)
     run_command("%s -f %s%s" % (make, make_file, make_target))
 
     if not os.access(exe_file, os.X_OK):
-        sipconfig.error("Failed to create %s using platform %s. Make sure your compiler is installed correctly and you have a working %s on your PATH." % (exe_file, sipcfg.platform, make))
+        sipconfig.error("Failed to determine the layout of your Qt installtion. Try again using the --verbose flag to see more detail about the problem.")
 
     # Create the output file, first making sure it doesn't exist.
     remove_file(out_file)
@@ -1715,7 +1582,7 @@ int main(int, char **)
     f.close()
 
     global qt_dir, qt_incdir, qt_libdir, qt_bindir, qt_datadir, qt_pluginsdir
-    global qt_shared, qt_xfeatures
+    global qt_version, qt_edition, qt_shared, qt_xfeatures
 
     qt_dir = lines[0]
     qt_incdir = lines[1]
@@ -1723,8 +1590,34 @@ int main(int, char **)
     qt_bindir = lines[3]
     qt_datadir = lines[4]
     qt_pluginsdir = lines[5]
-    qt_shared = lines[6]
-    qt_xfeatures = lines[7:]
+    qt_version = lines[6]
+    qt_edition = lines[7]
+    qt_shared = lines[8]
+    qt_xfeatures = lines[9:]
+
+    try:
+        qt_version = int(qt_version)
+    except ValueError:
+        sipconfig.error("QT_VERSION should be a number but \"%s\" was found." % qt_version)
+
+    try:
+        qt_edition = int(qt_edition)
+    except ValueError:
+        sipconfig.error("QT_EDITION should be a number but \"%s\" was found." % qt_edition)
+
+    # Now convert the edition to a descriptive string.  The order of testing is
+    # important.
+    if qt_edition & 0x200:
+        # It has ActiveQt.
+        qt_edition = "Desktop"
+    elif qt_edition & 0x008:
+        # It has OpenGL.
+        qt_edition = "free"
+    elif qt_edition & 0x002:
+        # It has GUI.
+        qt_edition = "Desktop Light"
+    else:
+        qt_edition = "Console"
 
     if qt_shared:
         if opts.staticplugins:
@@ -1743,7 +1636,7 @@ def main(argv):
     argv is the list of command line arguments.
     """
     # Check SIP is new enough.
-    if sipcfg.sip_version_str[:8] != "snapshot":
+    if "snapshot" not in sipcfg.sip_version_str:
         if sipcfg.sip_version < sip_min_version:
             sipconfig.error("This version of PyQt requires SIP v%s or later" % sipconfig.version_to_string(sip_min_version))
 
