@@ -1,5 +1,5 @@
 /**********************************************************************
-** Copyright (C) 2002-2005 Detlev Offenbach <detlev@die-offenbachs.de>
+** Copyright (C) 2002-2007 Detlev Offenbach <detlev@die-offenbachs.de>
 **
 ** This is a modified version of lupdate. The original is part of Qt-Linguist.
 ** The copyright of the original file can be found below.
@@ -62,7 +62,7 @@ enum { Tok_Eof, Tok_class, Tok_return, Tok_tr,
        Tok_trUtf8, Tok_translate, Tok_Ident,
        Tok_Comment, Tok_Dot, Tok_String,
        Tok_LeftParen, Tok_RightParen,
-       Tok_Comma, Tok_None};
+       Tok_Comma, Tok_None, Tok_Integer};
 
 /*
   The tokenizer maintains the following global variables. The names
@@ -76,6 +76,7 @@ static char yyComment[65536];
 static size_t yyCommentLen;
 static char yyString[65536];
 static size_t yyStringLen;
+static qlonglong yyInteger;
 static QStack<int> yySavedParenDepth;
 static int yyParenDepth;
 static int yyLineNo;
@@ -100,7 +101,7 @@ static bool yyParsingUtf8;
 static int getCharFromFile()
 {
     int c;
-    
+
     if ( buf < 0 )
         c = getc( yyInFile );
     else {
@@ -230,18 +231,18 @@ static int getToken()
                     int quoteChar;
                     int trippelQuote, singleQuote;
                     int in;
-                
+
                     quoteChar = yyCh;
                     trippelQuote = 0;
                     singleQuote = 1;
                     in = 0;
                     yyCh = getChar();
                     quiet = false;
-                    
+
                     while ( yyCh != EOF ) {
                         if ( singleQuote && (yyCh == '\n' || (in && yyCh == quoteChar)) )
                             break;
-                        
+
                         if ( yyCh == quoteChar ) {
                             if (peekChar() == quoteChar) {
                                 yyCh = getChar();
@@ -266,10 +267,10 @@ static int getToken()
                                 break;
                         } else
                             in = 1;
-                        
+
                         if ( yyCh == '\\' ) {
                             yyCh = getChar();
-                            
+
                             if ( yyCh == 'x' ) {
                                 QByteArray hex = "0";
 
@@ -288,7 +289,7 @@ static int getToken()
                             } else if ( yyCh >= '0' && yyCh < '8' ) {
                                 QByteArray oct = "";
                                 int n = 0;
-    
+
                                 do {
                                     oct += (char) yyCh;
                                     ++n;
@@ -321,7 +322,7 @@ static int getToken()
                                 yyCh = getChar();
                             } else {
                                 QByteArray originalBytes;
-                                while ( yyCh != EOF && yyCh != '\n' && yyCh != '"' && yyCh != '\\' ) {
+                                while ( yyCh != EOF && (trippelQuote || yyCh != '\n') && yyCh != quoteChar && yyCh != '\\' ) {
                                     if ( yyParsingUtf8 && yyCh >= 0x80 && !quiet) {
                                         qWarning( "%s:%d: Non-ASCII character detected in trUtf8 string",
                                                 (const char *) yyFileName, yyLineNo );
@@ -330,10 +331,10 @@ static int getToken()
                                     originalBytes += (char)yyCh;
                                     yyCh = getChar();
                                 }
-    
+
                                 QString unicodeStr = yyCodecForSource->toUnicode(originalBytes);
                                 QByteArray convertedBytes;
-    
+
                                 if (!yyCodecForTr->canEncode(unicodeStr) && !quiet) {
                                     qWarning( "%s:%d: Cannot convert Python string from %s to %s",
                                               (const char *) yyFileName, yyLineNo, yyCodecForSource->name().constData(),
@@ -341,7 +342,7 @@ static int getToken()
                                     quiet = true;
                                 }
                                 convertedBytes = yyCodecForTr->fromUnicode(unicodeStr);
-    
+
                                 size_t len = qMin((size_t)convertedBytes.size(), sizeof(yyString) - yyStringLen - 1);
                                 memcpy(yyString + yyStringLen, convertedBytes.constData(), len);
                                 yyStringLen += len;
@@ -355,7 +356,7 @@ static int getToken()
                         qWarning( "%s:%d: Unterminated string",
                           (const char *) yyFileName, yyLineNo );
                     }
-                        
+
                     if ( yyCh == EOF ) {
                         return Tok_Eof;
                     } else {
@@ -381,6 +382,34 @@ static int getToken()
                 case '.':
                     yyCh = getChar();
                     return Tok_Dot;
+                case '0':
+                case '1':
+                case '2':
+                case '3':
+                case '4':
+                case '5':
+                case '6':
+                case '7':
+                case '8':
+                case '9':
+                    {
+                        QByteArray ba;
+                        ba+=yyCh;
+                        yyCh = getChar();
+                        bool hex = yyCh == 'x';
+                        if ( hex ) {
+                            ba+=yyCh;
+                            yyCh = getChar();
+                        }
+                        while ( (hex ? isxdigit(yyCh) : isdigit(yyCh)) ) {
+                            ba+=yyCh;
+                            yyCh = getChar();
+                        }
+                        bool ok;
+                        yyInteger = ba.toLongLong(&ok);
+                        if (ok) return Tok_Integer;
+                        break;
+                    }
                 default:
                     yyCh = getChar();
             }
@@ -393,10 +422,10 @@ static int getToken()
   The second part of this source file is the parser. It accomplishes
   a very easy task: It finds all strings inside a tr() or translate()
   call, and possibly finds out the context of the call. It supports
-  three cases: 
-  (1) the context is specified, as in FunnyDialog.tr("Hello") or 
+  three cases:
+  (1) the context is specified, as in FunnyDialog.tr("Hello") or
      translate("FunnyDialog", "Hello");
-  (2) the call appears within an inlined function; 
+  (2) the call appears within an inlined function;
   (3) the call appears within a function defined outside the class definition.
 */
 
@@ -426,22 +455,22 @@ static bool matchEncoding( bool *utf8 )
     // Remove any leading module paths.
     if (yyTok == Tok_Ident && strcmp(yyIdent, "PyQt4") == 0)
     {
-	yyTok = getToken();
+    yyTok = getToken();
 
-	if (yyTok != Tok_Dot)
-	    return false;
+    if (yyTok != Tok_Dot)
+        return false;
 
-	yyTok = getToken();
+    yyTok = getToken();
     }
 
     if (yyTok == Tok_Ident && (strcmp(yyIdent, "QtGui") == 0 || strcmp(yyIdent, "QtCore") == 0))
     {
-	yyTok = getToken();
+    yyTok = getToken();
 
-	if (yyTok != Tok_Dot)
-	    return false;
+    if (yyTok != Tok_Dot)
+        return false;
 
-	yyTok = getToken();
+    yyTok = getToken();
     }
 
     if ( yyTok == Tok_Ident ) {
@@ -464,11 +493,52 @@ static bool matchStringOrNone(QByteArray *s)
     bool matches = matchString(s);
 
     if (!matches)
-	matches = match(Tok_None);
+    matches = match(Tok_None);
 
     return matches;
 }
 
+/*
+ * match any expression that can return a number, which can be
+ * 1. Literal number (e.g. '11')
+ * 2. simple identifier (e.g. 'm_count')
+ * 3. simple function call (e.g. 'size()' )
+ * 4. function call on an object (e.g. 'list.size()')
+ *
+ * Other cases:
+ * size(2,4)
+ * list().size()
+ * list(a,b).size(2,4)
+ * etc...
+ */
+static bool matchExpression()
+{
+    if (match(Tok_Integer)) {
+        return true;
+    }
+
+    int parenlevel = 0;
+    while (match(Tok_Ident) || parenlevel > 0) {
+        if (yyTok == Tok_RightParen) {
+            if (parenlevel == 0) break;
+            --parenlevel;
+            yyTok = getToken();
+        } else if (yyTok == Tok_LeftParen) {
+            yyTok = getToken();
+            if (yyTok == Tok_RightParen) {
+                yyTok = getToken();
+            } else {
+                ++parenlevel;
+            }
+        } else if (yyTok == Tok_Ident) {
+            continue;
+        } else if (parenlevel == 0) {
+            return false;
+        }
+    }
+    return true;
+}
+
 static void parse( MetaTranslator *tor, const char *initialContext,
            const char *defaultContext )
 {
@@ -490,45 +560,99 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                 break;
             case Tok_tr:
             case Tok_trUtf8:
-                utf8 = ( yyTok == Tok_trUtf8 );
+                utf8 = (yyTok == Tok_trUtf8);
                 yyTok = getToken();
-                if ( match(Tok_LeftParen) && matchString(&text) ) {
+                if (match(Tok_LeftParen) && matchString(&text))
+                {
                     com = "";
-                    if ( match(Tok_RightParen) || (match(Tok_Comma) &&
-                        matchStringOrNone(&com) && match(Tok_RightParen)) ) {
-                        if ( prefix.isNull() ) {
-                            context = defaultContext;
-                        } else if ( qstrcmp(prefix, "self") == 0 ) {
-                            context = functionContext;
-                        } else {
-                            context = prefix;
+                    bool plural = false;
+
+                    if (match(Tok_RightParen))
+                    {
+                        // There is no comment or plural arguments.
+                    }
+                    else if (match(Tok_Comma) && matchStringOrNone(&com))
+                    {
+                        // There is a comment argument.
+                        if (match(Tok_RightParen))
+                        {
+                            // There is no plural argument.
                         }
-                        prefix = (const char *) 0;
-                        
-                        if ( qualifiedContexts.contains(context) )
-                            context = qualifiedContexts[context];
-                        tor->insert( MetaTranslatorMessage(context, text, com,
-                               QString::null, utf8) );
+                        else if (match(Tok_Comma))
+                        {
+                            // There is a plural argument.
+                            plural = true;
+                        }
+                    }
+
+                    if (prefix.isNull())
+                        context = defaultContext;
+                    else if (qstrcmp(prefix, "self") == 0)
+                        context = functionContext;
+                    else
+                        context = prefix;
+
+                    prefix = (const char *) 0;
+
+                    if (qualifiedContexts.contains(context))
+                        context = qualifiedContexts[context];
+
+                    if (!text.isEmpty())
+                    {
+                        tor->insert(MetaTranslatorMessage(context, text, com, 
+                                yyFileName, yyLineNo,
+                                QStringList(), utf8,
+                                MetaTranslatorMessage::Unfinished, plural));
                     }
                 }
                 break;
             case Tok_translate:
-            utf8 = false;
+                utf8 = false;
                 yyTok = getToken();
                 if ( match(Tok_LeftParen) &&
                  matchString(&context) &&
                  match(Tok_Comma) &&
                  matchString(&text) ) {
                     com = "";
-                    if ( match(Tok_RightParen) ||
-                     (match(Tok_Comma) &&
-                      matchStringOrNone(&com) &&
-                      (match(Tok_RightParen) ||
-                       match(Tok_Comma) &&
-                       matchEncoding(&utf8) &&
-                       match(Tok_RightParen))) )
-                        tor->insert( MetaTranslatorMessage(context, text, com,
-                                                       QString(), utf8) );
+                    bool plural = false;
+                    if (!match(Tok_RightParen)) {
+                        // look for comment
+                        if ( match(Tok_Comma) && matchStringOrNone(&com)) {
+                            if (!match(Tok_RightParen)) {
+                                // look for encoding
+                                if (match(Tok_Comma)) {
+                                    if (matchEncoding(&utf8)) {
+                                        if (!match(Tok_RightParen)) {
+                                            // look for the plural quantifier,
+                                            // this can be a number, an identifier or a function call,
+                                            // so for simplicity we mark it as plural if we know we have a comma instead of an
+                                            // right parentheses.
+                                            plural = match(Tok_Comma);
+                                        }
+                                    } else {
+                                        // This can be a QTranslator::translate("context", "source", "comment", n) plural translation
+                                        if (matchExpression() && match(Tok_RightParen)) {
+                                            plural = true;
+                                        } else {
+                                            break;
+                                        }
+                                    }
+                                } else {
+                                    break;
+                                }
+                            }
+                        } else {
+                            break;
+                        }
+                    }
+                    if (!text.isEmpty())
+                    {
+                        tor->insert( MetaTranslatorMessage(context, text, com, 
+                                                        yyFileName, yyLineNo,
+                                                        QStringList(), utf8,
+                                                        MetaTranslatorMessage::Unfinished,
+                                                        plural) );
+                    }
                 }
                 break;
             case Tok_Ident:
@@ -541,29 +665,30 @@ static void parse( MetaTranslator *tor, const char *initialContext,
                 break;
             case Tok_Comment:
                 com = yyComment;
-            com = com.simplified();
-            if ( com.left(sizeof(MagicComment) - 1) == MagicComment ) {
-                com.remove( 0, sizeof(MagicComment) - 1 );
-                int k = com.indexOf( ' ' );
-                if ( k == -1 ) {
-                    context = com;
-                } else {
-                    context = com.left( k );
-                    com.remove( 0, k + 1 );
-                    tor->insert( MetaTranslatorMessage(context, "", com,
-                                                       QString(), false) );
-
+                com = com.simplified();
+                if ( com.left(sizeof(MagicComment) - 1) == MagicComment ) {
+                    com.remove( 0, sizeof(MagicComment) - 1 );
+                    int k = com.indexOf( ' ' );
+                    if ( k == -1 ) {
+                        context = com;
+                    } else {
+                        context = com.left( k );
+                        com.remove( 0, k + 1 );
+                        tor->insert( MetaTranslatorMessage(context, "", com,
+                                                        yyFileName, yyLineNo,
+                                                        QStringList(), false) );
+    
+                    }
                 }
-            }
-            yyTok = getToken();
-            break;
+                yyTok = getToken();
+                break;
             default:
                 yyTok = getToken();
         }
     }
 
     if ( yyParenDepth != 0 )
-        qWarning( "%s: Unbalanced parentheses in Python code", 
+        qWarning( "%s: Unbalanced parentheses in Python code",
             (const char *) yyFileName );
 }
 
@@ -576,7 +701,7 @@ void fetchtr_py( const char *fileName, MetaTranslator *tor,
             char buf[100];
             strerror_s(buf, sizeof(buf), errno);
             fprintf( stderr,
-                     "pylupdate error: Cannot open Python source file '%s': %s\n",
+                     "pylupdate4 error: Cannot open Python source file '%s': %s\n",
                      fileName, buf );
         }
 #else
@@ -584,7 +709,7 @@ void fetchtr_py( const char *fileName, MetaTranslator *tor,
     if ( yyInFile == 0 ) {
         if ( mustExist )
             fprintf( stderr,
-                     "pylupdate error: Cannot open Python source file '%s': %s\n",
+                     "pylupdate4 error: Cannot open Python source file '%s': %s\n",
                      fileName, strerror(errno) );
 #endif
         return;
@@ -609,6 +734,11 @@ public:
     virtual bool characters( const QString& ch );
     virtual bool fatalError( const QXmlParseException& exception );
 
+    virtual void setDocumentLocator(QXmlLocator *locator)
+    {
+        m_locator = locator;
+    }
+    QXmlLocator *m_locator;
 private:
     void flush();
 
@@ -619,7 +749,7 @@ private:
     QString comment;
 
     QString accum;
-
+    int m_lineNumber;
     bool trString;
 };
 
@@ -642,6 +772,7 @@ bool UiHandler::startElement( const QString& /* namespaceURI */,
             trString = false;
         }
     }
+    if (trString) m_lineNumber = m_locator->lineNumber();
     accum.truncate( 0 );
     return true;
 }
@@ -686,8 +817,8 @@ void UiHandler::flush()
 {
     if ( !context.isEmpty() && !source.isEmpty() )
         tor->insert( MetaTranslatorMessage(context.toUtf8(), source.toUtf8(),
-                                           comment.toUtf8(), QString(),
-                                           true) );
+                                           comment.toUtf8(), QString(fname), m_lineNumber,
+                                           QStringList(), true) );
     source.truncate( 0 );
     comment.truncate( 0 );
 }
@@ -697,17 +828,17 @@ void fetchtr_ui( const char *fileName, MetaTranslator *tor,
 {
     QFile f( fileName );
     if ( !f.open(QIODevice::ReadOnly) ) {
-		if ( mustExist ) {
+        if ( mustExist ) {
 #if defined(_MSC_VER) && _MSC_VER >= 1400
-			char buf[100];
-			strerror_s(buf, sizeof(buf), errno);
-			fprintf( stderr, "lupdate error: cannot open UI file '%s': %s\n",
+            char buf[100];
+            strerror_s(buf, sizeof(buf), errno);
+            fprintf( stderr, "pylupdate4 error: cannot open UI file '%s': %s\n",
                      fileName, buf );
 #else
-            fprintf( stderr, "lupdate error: cannot open UI file '%s': %s\n",
+            fprintf( stderr, "pylupdate4 error: cannot open UI file '%s': %s\n",
                      fileName, strerror(errno) );
 #endif
-		}
+        }
         return;
     }
 
