@@ -196,7 +196,23 @@ class UIParser(object):
             if type(widget) == QtGui.QWidget:
                 self.stack.topwidget.setCentralWidget(widget)
             elif isinstance(widget, QtGui.QToolBar):
-                self.stack.topwidget.addToolBar(widget)
+                tbArea = self.wprops.getAttribute(elem, "toolBarArea")
+
+                if tbArea is None:
+                    self.stack.topwidget.addToolBar(widget)
+                else:
+                    if isinstance(tbArea, basestring):
+                        tbArea = getattr(QtCore.Qt, tbArea)
+                    else:
+                        tbArea = QtCore.Qt.ToolBarArea(tbArea)
+
+                    self.stack.topwidget.addToolBar(tbArea, widget)
+
+                tbBreak = self.wprops.getAttribute(elem, "toolBarBreak")
+
+                if tbBreak:
+                    self.stack.topwidget.insertToolBarBreak(widget)
+
             elif isinstance(widget, QtGui.QMenuBar):
                 self.stack.topwidget.setMenuBar(widget)
             elif isinstance(widget, QtGui.QStatusBar):
